// Jrn Loviscach 2001-05-26

/*
Mens, Untermens, Pop-Up-Mens, Rahmen um Komponenten (hier ein Button), Icons, Zeichenflchen
*/

package menusandbuttons;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class MenusAndButtonsDemo
{
  JFrame myFrame; // unser Fenster
  JLabel myLabel; // unsere Ausgabe-Textzeile
  DrawPanel myPanel; // unser Zeichenfeld
  Controller myController; // wertet Events zentral aus

  public static void main(String[] args)
  {
    new MenusAndButtonsDemo();
  }

  MenusAndButtonsDemo()
  {
    myController = new Controller(this);
    // Dank this knnen wir aus Controller auf die Elemente von MenusAndButtonsDemo zugreifen.

    myFrame = new JFrame("MenusAndButtonsDemo");
    myFrame.addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){System.exit(0);}});

    myFrame.setSize(300, 300);
    myFrame.setLocation(100, 100);

    Container myPane = myFrame.getContentPane();
    myPane.setLayout(new BorderLayout());

    /******** Mens bauen ********/

    JMenu myMenuName = new JMenu("Name");
    myMenuName.setMnemonic(KeyEvent.VK_N); // Alt+n klappt Men auf.

    JMenuItem a = new JMenuItem("Alice");
    a.setActionCommand("Alice");
    a.addActionListener(myController);
    myMenuName.add(a);

    JMenuItem b = new JMenuItem("Bert");
    b.setActionCommand("Bert");
    b.addActionListener(myController);
    b.setMnemonic(KeyEvent.VK_B); // Alt+b ruft Funktion auf, wenn Men aufgeklappt.
    myMenuName.add(b);

    myMenuName.addSeparator();

    JMenuItem c = new JMenuItem("Chris");
    c.setActionCommand("Chris");
    c.addActionListener(myController);
    c.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, Event.CTRL_MASK));
    // Strg+c ruft Funktion auch bei geschlossenem Men auf.
    myMenuName.add(c);

    JMenu myMenuExtras = new JMenu("Extras");

    JMenuItem s = new JMenuItem("Sonne");
    s.setActionCommand("Sonne");
    s.addActionListener(myController);
    myMenuExtras.add(s);

    JMenu myMenuRegen = new JMenu("Regen"); // wird ein Untermen

    JMenuItem e = new JMenuItem("Ergiebig");
    e.setActionCommand("Ergiebig");
    e.addActionListener(myController);
    myMenuRegen.add(e);

    JMenuItem g = new JMenuItem("Gelegentlich");
    g.setActionCommand("Gelegentlich");
    g.addActionListener(myController);
    myMenuRegen.add(g);

    myMenuExtras.add(myMenuRegen); // Untermen einbinden

    JMenuBar myMenuBar = new JMenuBar(); // nimmt Mens auf
    myMenuBar.add(myMenuName);
    myMenuBar.add(myMenuExtras);
    myFrame.setJMenuBar(myMenuBar);

    /******** Ausgabefeld bauen ********/

    myLabel = new JLabel("---");
    myPane.add(myLabel, BorderLayout.SOUTH);

    /******** Buttons bauen ********/

    Container myBox = Box.createVerticalBox();
    myPane.add(myBox, BorderLayout.WEST);

    JButton bA = new JButton("<html><B>A</B> mit <I>HTML</I>");
    bA.setActionCommand("A");
    bA.addActionListener(myController);
    myBox.add(bA);

    JButton bB = new JButton("B");
    bB.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
    // Rahmen so, dass Button aus der Flche herausragt
    bB.setActionCommand("B");
    bB.addActionListener(myController);
    myBox.add(bB);

    JButton bC = new JButton("C");
    bC.setBorder(BorderFactory.createLineBorder(new Color(0, 0, 255), 4));
    // Rahmen = blaue Linie
    bC.setActionCommand("C");
    bC.addActionListener(myController);
    myBox.add(bC);

    JButton bD = new JButton("D"); // ruft Popup-Men auf
    final JPopupMenu myPopup = new JPopupMenu();
    JMenuItem myPopupItem1 = new JMenuItem("1");
    myPopupItem1.setActionCommand("1");
    myPopupItem1.addActionListener(myController);
    myPopup.add(myPopupItem1);
    JMenuItem myPopupItem2 = new JMenuItem("2");
    myPopupItem2.setActionCommand("2");
    myPopupItem2.addActionListener(myController);
    myPopup.add(myPopupItem2);
    bD.addMouseListener(
      new MouseAdapter()
      {
        public void mousePressed(MouseEvent e)
        {
          myPopup.show(e.getComponent(), e.getX(), e.getY());
        }
      }
    );
    myBox.add(bD);

    JButton bE = new JButton();
    Icon myIcon = new ImageIcon("C:/windows/desktop/icon1.gif");
    bE.setIcon(myIcon);
    bE.setMaximumSize(new Dimension(myIcon.getIconWidth(), myIcon.getIconHeight()));
    bE.setActionCommand("E");
    bE.addActionListener(myController);
    myBox.add(bE);

    JButton bF = new JButton();
    myIcon = new ImageIcon("C:/windows/desktop/icon.gif");
    bF.setIcon(myIcon);
    bF.setMaximumSize(new Dimension(myIcon.getIconWidth(), myIcon.getIconHeight()));
    bF.setPressedIcon(new ImageIcon("C:/windows/desktop/pressed.gif"));
    // GIF-Animation geht auch!
    bF.setRolloverIcon(new ImageIcon("C:/windows/desktop/over.gif"));
    bF.setActionCommand("F");
    bF.addActionListener(myController);
    myBox.add(bF);

    /******** Zeichenfeld bauen ********/

    myPanel = new DrawPanel(this);
    // Dank this knnen wir aus DrawPanel auf die Elemente von MenusAndButtonsDemo zugreifen.
    myPane.add(myPanel, BorderLayout.CENTER);

    /******** anzeigen ********/

    myFrame.setVisible(true);
  }
}

class Controller
implements ActionListener
{
  MenusAndButtonsDemo myGui;

  Controller(MenusAndButtonsDemo m)
  {
    myGui = m;
  }

  public void actionPerformed (ActionEvent e)
  {
    myGui.myLabel.setText("letzte Aktion: " + e.getActionCommand());
    myGui.myPanel.repaint();
    // sonst bei Bettigen der Buttons keine Aktualisierung der Zeichenflche
  }
}

class DrawPanel
extends JPanel
{
  MenusAndButtonsDemo myGui;

  DrawPanel(MenusAndButtonsDemo m)
  {
    super();
    myGui = m;
  }

  public void paint(Graphics g)
  {
    super.paint(g);
    for( int i = 0; i < 10; ++i)
    {
      g.setColor(new Color(255, 25*i, 255-25*i));
      g.drawString(myGui.myLabel.getText(),10+10*i,10+10*i);
      g.drawOval(10*i,10*i,10+20*i,10+20*i);
    }
  }
}