// Jrn Loviscach 2001-05-26

/*
Objektserialisierung ist in Java 2 Platform eingebaut.
*/

package serialize;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;

public class SerializeDemo
{
  Data myData; // die zu sichernden Daten
  JFrame myFrame; // unser Fenster
  JTextField myTextField1; // oberes Textfeld
  JTextField myTextField2; // unteres Textfeld

  public static void main(String[] args)
  {
    new SerializeDemo(); // Eine einfache Art, OO-Programme zu bauen: main konstruiert einfach ein Anwendungsobjekt.
  }                      // Die Logik steckt dann im Konstruktor.

  SerializeDemo()
  {
    myData = new Data();

    myFrame = new JFrame("SerializeDemo");
    myFrame.addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){System.exit(0);}});

    myFrame.setSize(300, 300);
    myFrame.setLocation(100, 100);

    Container myPane = myFrame.getContentPane();
    myPane.setLayout(new GridLayout(0, 1));

    myTextField1 = new JTextField();
    myPane.add(myTextField1);

    myTextField2 = new JTextField();
    myPane.add(myTextField2);

    JButton bStore = new JButton("In internen Speicher bernehmen");
    myPane.add(bStore);

    JButton bRetrieve = new JButton("Aus internem Speicher holen");
    myPane.add(bRetrieve);

    JButton bSave = new JButton("Auf Platte speichern");
    myPane.add(bSave);

    JButton bOpen = new JButton("Von Platte laden");
    myPane.add(bOpen);

    // Events verdrahten
    bStore.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          myData.text1 = myTextField1.getText();
          myData.text2 = myTextField2.getText();
        }
      }
    );

    bRetrieve.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          myTextField1.setText(myData.text1);
          myTextField2.setText(myData.text2);
        }
      }
    );

    bSave.addActionListener( new SaveDialog(this) );

    bOpen.addActionListener( new OpenDialog(this) );

    myFrame.setVisible(true);
  }
}

class Data // soll auf Platte gesichert werden
implements Serializable // ist ein leeres Interface, dient Java nur zur Kenzeichnung
{
  String text1; // Auch referenzierte Elemente werden gesichert! (Rekursion)
  String text2;
  // Ggf. als "transient" markierte Elemente werden nicht gesichert.
}

class SaveDialog
implements ActionListener
{
  SerializeDemo myParent;

  SaveDialog(SerializeDemo parent)
  {
    myParent = parent;
  }

  public void actionPerformed(ActionEvent ae)
  {
    JFileChooser myFileChooser = new JFileChooser();
    int result = myFileChooser.showSaveDialog(myParent.myFrame); // Dialog blockiert myFrame und liegt darber.
    if (result == JFileChooser.CANCEL_OPTION)
    {
      return;
    }
    try
    {
      FileOutputStream myOutFile = new FileOutputStream(myFileChooser.getSelectedFile());
      ObjectOutputStream myOutputStream = new ObjectOutputStream(myOutFile);
      myOutputStream.writeObject(myParent.myData); // brigens auch mehrere Ausgaben hintereinander mglich
      myOutputStream.flush();
      myOutFile.close();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog(myParent.myFrame, "Fehler beim Speichern:\n" + e,
                                                      "Dateifehler", JOptionPane.WARNING_MESSAGE);
    }
  }
}

class OpenDialog
implements ActionListener
{
  SerializeDemo myParent;

  OpenDialog(SerializeDemo parent)
  {
    myParent = parent;
  }

  public void actionPerformed(ActionEvent ae)
  {
    JFileChooser myFileChooser = new JFileChooser();
    int result = myFileChooser.showOpenDialog(myParent.myFrame); // Dialog blockiert myFrame und liegt darber.
    if (result == JFileChooser.CANCEL_OPTION)
    {
      return;
    }

    Data d;
    try
    {
      FileInputStream myInFile = new FileInputStream(myFileChooser.getSelectedFile());
      ObjectInputStream myInputStream = new ObjectInputStream(myInFile);
      d = (Data) myInputStream.readObject(); // brigens auch mehrere Ausgaben hintereinander mglich
      myInFile.close();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog(myParent.myFrame, "Fehler beim Laden:\n" + e,
                                                      "Dateifehler", JOptionPane.WARNING_MESSAGE);
      return;
    }

    myParent.myData = d;
    myParent.myTextField1.setText(d.text1);
    myParent.myTextField2.setText(d.text2);
  }
}